@@ -363,11 +363,14 @@ process_line(char *line, FILE *b, FILE *c, FILE *d)
                 inhlink = TRUE;
                 /* remove trailing newline etc */
                 str = line + strlen(line) - 1;
-                while (*str=='\r' || *str=='\n') *str-- = NUL;
+                while (*str == '\r' || *str == '\n') *str-- = NUL;
                 fprintf(b, "%s", line + 2);
             } else if (inhlink) {
                 inhlink = FALSE;
 	        fputs(line + 2, b);	/* copy directly */
+	    } else if (strncmp(line, "^figure_", 8) == 0) {
+		/* fall through to embedded figure as in doc2web */
+		;
             } else {
                 if (line[2] == '!') { /* hack for function sections */
                     const char magic[] = "<!-- INCLUDE_NEXT_TABLE -->";
@@ -377,8 +380,13 @@ process_line(char *line, FILE *b, FILE *c, FILE *d)
                 inhlink = FALSE;
 	        fputs(line + 1, b);	/* copy directly */
             }
-	    break;		/* ignore */
+	    if (strncmp(line, "^figure_", 8) == 0)
+		; /* Fall through to embedded figure */
+	    else
+		break;
 	}
+    case 'D':			/* link to online demo figure */
+	    break;
     case 'F':			/* latex embedded figure */
             if (para) fprintf(b, "</p><p align=\"justify\">\n");
             fprintf(b, "<img src=\"%s.png\" alt=\"%s\">\n", line2+1, line2+1);
@@ -399,7 +407,7 @@ process_line(char *line, FILE *b, FILE *c, FILE *d)
 		    /* Continuation of bulleted line */
 		    fputs(line2+2, b);
 		}
-		else {
+		else if (strncmp(line+1, "TeX", 3) != 0) {
 		    if (strchr(line, '\n'))
 			*(strchr(line, '\n')) = '\0';
 		    fprintf(b, "<li><pre>%s</pre>\n", line + 1);
