@@ -1,7 +1,3 @@
-/*
- * $Id: breaders.h,v 1.4 2008/03/14 02:56:22 sfeam Exp $
- */
-
 /* GNUPLOT - binedf.h */
 
 /*[
@@ -46,13 +42,16 @@
 #endif
 
 #include "syscfg.h"
+#include "gadgets.h"
 
 /* Prototypes of functions exported by breaders.c */
 
-void edf_filetype_function __PROTO((void));
-void png_filetype_function __PROTO((void));
-void gif_filetype_function __PROTO((void));
-void jpeg_filetype_function __PROTO((void));
-int  df_libgd_get_pixel __PROTO((int i, int j, int component));
+void edf_filetype_function(void);
+void png_filetype_function(void);
+void gif_filetype_function(void);
+void jpeg_filetype_function(void);
+int  df_libgd_get_pixel(int i, int j, int component);
+
+TBOOLEAN df_read_pixmap(t_pixmap *pixmap);
 
 #endif /* GNUPLOT_BINEDF_H */
