@@ -145,6 +145,7 @@ void restore_prompt(void);
 #define bind_command()
 #endif
 void array_command(void);
+void local_array_command(int locality);
 void break_command(void);
 void call_command(void);
 void changedir_command(void);
@@ -173,6 +174,7 @@ void printerr_command(void);
 void pwd_command(void);
 void refresh_request(void);
 void refresh_command(void);
+void remultiplot_command(void);
 void replot_command(void);
 void reread_command(void);
 void return_command(void);
@@ -193,6 +195,8 @@ void extend_input_line(void);
 void extend_token_table(void);
 int com_line(void);
 int do_line(void);
+int preprocess_line(void);
+int step_through_line(void);
 void do_string(const char* s);
 void do_string_and_free(char* s);
 TBOOLEAN iteration_early_exit(void);
@@ -204,7 +208,6 @@ void do_string_replot(const char* s);
 void define(void);
 
 void replotrequest(void); /* used in command.c & mouse.c */
-void shadow_one_variable(struct udvt_entry *udv);
 
 void print_set_output(char *, TBOOLEAN, TBOOLEAN); /* set print output file */
 char *print_show_output(void); /* show print output file */
