@@ -215,6 +215,11 @@ static TBOOLEAN setting_zoom_region = FALSE;
  * coordinate system */
 static int setting_zoom_x, setting_zoom_y;
 
+/* flag to indicate that in-line axis ranges should be ignored
+ * and zoom/pan range limits take precedence over auto-scaling
+ */
+static TBOOLEAN inside_zoom_flag;
+
 
 /* variables for changing the 3D view:
 */
@@ -265,10 +270,11 @@ static void alert(void);
 static void MousePosToGraphPosReal(int xx, int yy, double *x, double *y, double *x2, double *y2);
 static char *xy_format(void);
 static char *zoombox_format(void);
-static char *GetAnnotateString(char *s, double x, double y, int mode, char *fmt);
+static char *GetAnnotateString(char *s, size_t len, double x, double y, int mode, char *fmt);
 static char *xDateTimeFormat(double x, char *b, int mode);
 static void GetRulerString(char *p, double x, double y);
 static void apply_zoom(struct t_zoom * z);
+static void restore_saved_axis_structures(void);
 static void do_zoom(double xmin, double ymin, double x2min,
 		    double y2min, double xmax, double ymax, double x2max, double y2max);
 static void ZoomNext(void);
@@ -517,7 +523,7 @@ zoombox_format()
 /* formats the information for an annotation (middle mouse button clicked)
  */
 static char *
-GetAnnotateString(char *s, double x, double y, int mode, char *fmt)
+GetAnnotateString(char *s, size_t len, double x, double y, int mode, char *fmt)
 {
     if (axis_array[FIRST_X_AXIS].datatype == DT_DMS
     ||  axis_array[FIRST_Y_AXIS].datatype == DT_DMS) {
@@ -572,11 +578,11 @@ GetAnnotateString(char *s, double x, double y, int mode, char *fmt)
 	    r = rmin + x/cos(phi);
 
 	if (fmt)
-	    sprintf(s, fmt, theta, r);
+	    snprintf(s, len, fmt, theta, r);
 	else
 	    sprintf(s, "theta: %.1f%s  r: %g", theta, degree_sign, r);
     } else if ((mode == MOUSE_COORDINATES_ALT) && fmt) {
-	sprintf(s, fmt, x, y);	/* user defined format */
+	snprintf(s, len, fmt, x, y);	/* user defined format */
     } else if (mode == MOUSE_COORDINATES_FUNCTION) {
 	/* EXPERIMENTAL !!! */
 	t_value original_x, original_y;
@@ -599,7 +605,7 @@ GetAnnotateString(char *s, double x, double y, int mode, char *fmt)
 	gpfree_string(&readout);
     } else {
 	/* Default format ("set mouse mouseformat" is not active) */
-	sprintf(s, xy_format(), x, y);	/* usual x,y values */
+	snprintf(s, len, xy_format(), x, y);	/* usual x,y values */
     }
     return s + strlen(s);
 }
@@ -743,10 +749,11 @@ GetRulerString(char *p, double x, double y)
 
 static struct t_zoom *zoom_head = NULL, *zoom_now = NULL;
 static AXIS *axis_array_copy = NULL;
+static AXIS *shadow_axis_array_copy = NULL;
 
 /* Applies the zoom rectangle of  z  by sending the appropriate command
-   to gnuplot
-*/
+ * to gnuplot
+ */
 
 static void
 apply_zoom(struct t_zoom *z)
@@ -766,10 +773,19 @@ apply_zoom(struct t_zoom *z)
 
     /* The autoscale save/restore was too complicated, and broke refresh.
      * Just save the complete axis state and have done with it.
+     * FIXME: Can there be a memory leak if a non-linear axis was declared
+     *        inside the multiplot and not freed here before restoring the
+     *        original shadow axis structure?
      */
     if (zoom_now == zoom_head && z != zoom_head) {
 	axis_array_copy = gp_realloc( axis_array_copy, sizeof(axis_array), "axis_array copy");
 	memcpy(axis_array_copy, axis_array, sizeof(axis_array));
+	if (shadow_axis_array) {
+	    size_t shadowsize = NUMBER_OF_MAIN_VISIBLE_AXES * sizeof(AXIS);
+	    shadow_axis_array_copy =
+		gp_realloc( shadow_axis_array_copy, shadowsize, "shadow_array copy");
+	    memcpy(shadow_axis_array_copy, shadow_axis_array, shadowsize);
+	}
     }
 
     /* If we are zooming, we don't want to autoscale the range.
@@ -812,24 +828,7 @@ apply_zoom(struct t_zoom *z)
      * Apr 2015 - The same is now true (dynamic storage) for ticfmt, formatstring.
      */
     if (zoom_now == zoom_head) {
-	int i;
-	for (i=0; i<AXIS_ARRAY_SIZE; i++) {
-	    axis_array_copy[i].label = axis_array[i].label;
-	    axis_array_copy[i].ticdef.def.user = axis_array[i].ticdef.def.user;
-	    axis_array_copy[i].ticdef.font = axis_array[i].ticdef.font;
-	    axis_array_copy[i].ticfmt = axis_array[i].ticfmt;
-	    axis_array_copy[i].formatstring = axis_array[i].formatstring;
-	}
-	memcpy(axis_array, axis_array_copy, sizeof(axis_array));
-
-	/* The shadowed primary axis, if any, is not restored by the memcpy.	*/
-	/* We choose to recalculate the limits, but alternatively we could find	*/
-	/* some place to save/restore the unzoomed limits.			*/
-	if (nonlinear(&axis_array[FIRST_X_AXIS]))
-	    clone_linked_axes(&axis_array[FIRST_X_AXIS], axis_array[FIRST_X_AXIS].linked_to_primary);
-	if (nonlinear(&axis_array[FIRST_Y_AXIS]))
-	    clone_linked_axes(&axis_array[FIRST_Y_AXIS], axis_array[FIRST_Y_AXIS].linked_to_primary);
-
+	restore_saved_axis_structures();
 	/* Falling through to do_string_replot() does not work! */
 	if (volatile_data) {
 	    if (refresh_ok == E_REFRESH_OK_2D) {
@@ -841,23 +840,52 @@ apply_zoom(struct t_zoom *z)
 		return;
 	    }
 	}
-
     } else {
-	inside_zoom = TRUE;
+	inside_zoom_flag = TRUE;
     }
 
     do_string_replot("");
-    inside_zoom = FALSE;
+    inside_zoom_flag = FALSE;
+}
+
+/*
+ * helper routine called from apply_zoomi() and apply_saved_zoom()
+ */
+static void
+restore_saved_axis_structures()
+{
+    int i;
+    for (i=0; i<AXIS_ARRAY_SIZE; i++) {
+	axis_array_copy[i].label = axis_array[i].label;
+	axis_array_copy[i].ticdef.def.user = axis_array[i].ticdef.def.user;
+	axis_array_copy[i].ticdef.font = axis_array[i].ticdef.font;
+	axis_array_copy[i].ticfmt = axis_array[i].ticfmt;
+	axis_array_copy[i].formatstring = axis_array[i].formatstring;
+    }
+    memcpy(axis_array, axis_array_copy, sizeof(axis_array));
+    if (shadow_axis_array && shadow_axis_array_copy) {
+	size_t shadowsize = NUMBER_OF_MAIN_VISIBLE_AXES * sizeof(AXIS);
+	memcpy(shadow_axis_array, shadow_axis_array_copy, shadowsize);
+    }
 }
 
 
-/* makes a zoom: update zoom history, call gnuplot to set ranges + replot
-*/
+/* Execute zoom or pan:
+ * update zoom history, then call gnuplot to set ranges + replot
+ */
 
 static void
 do_zoom(double xmin, double ymin, double x2min, double y2min, double xmax, double ymax, double x2max, double y2max)
 {
     struct t_zoom *z;
+
+    /* Only one panel of a multiplot can be active for pan/zoom */
+    if (last_plot_was_multiplot && mouse_outside_active_region()) {
+	if (display_ipc_commands())
+	    fprintf(stderr, "(ignored) ");
+	return;
+    }
+
     if (zoom_head == NULL) {	/* queue not yet created, thus make its head */
 	zoom_head = gp_alloc(sizeof(struct t_zoom), "mouse zoom history head");
 	zoom_head->prev = NULL;
@@ -991,10 +1019,10 @@ UpdateStatuslineWithMouseSetting(mouse_setting_t * ms)
 	strcat(format, ms->fmt);
 	strcat(format, ", ");
 	strcat(format, ms->fmt);
-	sprintf(s0, format, surface_rot_x, surface_rot_z, surface_scale, surface_zscale);
+	snprintf(s0, 255, format, surface_rot_x, surface_rot_z, surface_scale, surface_zscale);
     } else if (!TICS_ON(axis_array[SECOND_X_AXIS].ticmode) && !TICS_ON(axis_array[SECOND_Y_AXIS].ticmode)) {
 	/* only first X and Y axis are in use */
-	sp = GetAnnotateString(s0, real_x, real_y, mouse_mode, mouse_alt_string);
+	sp = GetAnnotateString(s0, 255, real_x, real_y, mouse_mode, mouse_alt_string);
 	if (ruler.on)
 	    GetRulerString(sp, real_x, real_y);
     } else {
@@ -1320,9 +1348,15 @@ builtin_toggle_polardistance(struct gp_event_t *ge)
     if (!ge) {
 	return "`builtin-toggle-polardistance`";
     }
-    if (++mouse_setting.polardistance > 2) mouse_setting.polardistance = 0;
-	/* values: 0 (no polar coordinates), 1 (polar coordinates), 2 (tangent instead of angle) */
-    term->set_cursor((mouse_setting.polardistance ? -3:-4), ge->mx, ge->my); /* change cursor type */
+    /* values: 0 (no polar coordinates),
+     *         1 (polar coordinates),
+     *         2 (tangent instead of angle)
+     */
+    if (++mouse_setting.polardistance > 2)
+	mouse_setting.polardistance = 0;
+    /* change cursor type */
+    if (term->set_cursor)
+	term->set_cursor((mouse_setting.polardistance ? -3:-4), ge->mx, ge->my);
 # ifdef OS2
     PM_update_menu_items();
 # endif
@@ -2144,9 +2178,8 @@ event_buttonpress(struct gp_event_t *ge)
 	    /* the following variables are used to check if the box
 	     * is big enough to be considered as zoom box.
 	     */
-	    int dist_x = setting_zoom_x - mouse_x;
-	    int dist_y = setting_zoom_y - mouse_y;
-	    int dist = sqrt((double)(dist_x * dist_x + dist_y * dist_y));
+	    int dist_x = abs(setting_zoom_x - mouse_x);
+	    int dist_y = abs(setting_zoom_y - mouse_y);
 
 	    if (1 == b || 2 == b) {
 		/* zoom region is finished by the `wrong' button.
@@ -2165,7 +2198,7 @@ event_buttonpress(struct gp_event_t *ge)
 		}
 	    }
 
-	    if (dist > 10 /* more ore less arbitrary */ ) {
+	    if ((dist_x > 2 * term->tscale) && (dist_y > 2 * term->tscale)) {
 
 		double xmin, ymin, x2min, y2min;
 		double xmax, ymax, x2max, y2max;
@@ -2188,9 +2221,10 @@ event_buttonpress(struct gp_event_t *ge)
 		    fprintf(stderr, "zoom region finished.\n");
 		}
 	    } else {
-		/* silently ignore a tiny zoom box. This might
-		 * happen, if the user starts and finishes the
-		 * zoom box at the same position. */
+		/* silently ignore a tiny zoom box */
+		if (display_ipc_commands()) {
+		    fprintf(stderr, "tiny zoom region ignored.\n");
+		}
 	    }
 	    setting_zoom_region = FALSE;
 	}
@@ -2252,7 +2286,7 @@ event_buttonrelease(struct gp_event_t *ge)
 	     * only place, if the user didn't drag (rotate) the plot */
 
 	    if (!is_3d_plot || !motion) {
-		GetAnnotateString(s0, real_x, real_y, mouse_mode, mouse_alt_string);
+		GetAnnotateString(s0, 255, real_x, real_y, mouse_mode, mouse_alt_string);
 		term->set_clipboard(s0);
 		if (display_ipc_commands()) {
 		    fprintf(stderr, "put `%s' to clipboard.\n", s0);
@@ -2268,7 +2302,7 @@ event_buttonrelease(struct gp_event_t *ge)
 	    /* draw temporary annotation or label. For 3d plots this is
 	     * only done if the user didn't drag (scale) the plot */
 	    if (!is_3d_plot || !motion) {
-		GetAnnotateString(s0, real_x, real_y, mouse_mode, mouse_alt_string);
+		GetAnnotateString(s0, 255, real_x, real_y, mouse_mode, mouse_alt_string);
 		if (mouse_setting.label) {
 		    if (modifier_mask & Mod_Ctrl) {
 			remove_label(mouse_x, mouse_y);
@@ -2476,9 +2510,11 @@ event_reset(struct gp_event_t *ge)
     /* This hack is necessary on some systems in order to prevent one
      * character of input from being swallowed when the plot window is
      * closed. But which systems, exactly, and in what circumstances?
+     * The test against (void *)1 is to catch a call from set_terminal()
      */
     if (paused_for_mouse || !interactive) {
 	if (term && term_initialised
+	&&  (ge != (void *)1)
 	&&  (   !strncmp("x11",term->name,3)
 	     || !strncmp("wxt",term->name,3)
 	     || !strncmp("qt",term->name,2)))
@@ -2493,8 +2529,10 @@ event_reset(struct gp_event_t *ge)
 #endif
     }
 
-    /* Dummy up a keystroke event so that we can conveniently check for a  */
-    /* binding to "Close". We only get these for the current window. */
+    /* Dummy up a keystroke event so that we can conveniently check for a
+     * binding to "Close". We only get these for the current window.
+     * The test against (void *)1 is to catch a call from set_terminal()
+     */
     if (ge != (void *)1) {
 	ge->par1 = GP_Cancel;	/* Dummy keystroke */
 	ge->par2 = 0;		/* Not used; could pass window id here? */
@@ -2557,7 +2595,7 @@ do_event(struct gp_event_t *ge)
 	    break;
 	if (!strncmp(replot_line,"test",4))
 	    break;
-	if (multiplot)
+	if (in_multiplot)
 	    break;
 	do_string_replot("");
 	break;
@@ -3303,4 +3341,53 @@ void get_last_mouse_xy( double *x, double *y )
     *y = (y_axis == SECOND_Y_AXIS) ? real_y2 : real_y;
 }
 
+void
+zoom_reset_after_error()
+{
+    inside_zoom_flag = FALSE;
+    setting_zoom_region = FALSE;
+}
+
+TBOOLEAN
+inside_zoom()
+{
+    if (!inside_zoom_flag)
+	return FALSE;
+    if (in_multiplot
+    &&  multiplot_current_panel() > 0
+    &&  multiplot_current_panel() != multiplot_last_panel)
+	return FALSE;
+
+    return TRUE;
+}
+
+/* When replotting a multiplot interactively, the final (mouse is active) panel
+ * may have been zoomed but the previous panels overwrote the zoom state.
+ */
+void
+apply_saved_zoom()
+{
+    if (!zoom_now)
+	return;
+    if (1+multiplot_current_panel() != multiplot_last_panel)
+	return;
+
+    if (zoom_now == zoom_head) {
+	restore_saved_axis_structures();
+	return;
+    }
+
+    /* New range on primary axes */
+    set_explicit_range(&axis_array[FIRST_X_AXIS], zoom_now->xmin, zoom_now->xmax);
+    set_explicit_range(&axis_array[FIRST_Y_AXIS], zoom_now->ymin, zoom_now->ymax);
+
+    /* Do not mistake autoscaling initial state as an actual limit value. */
+    if (!is_3d_plot
+    && (zoom_now->x2min < VERYLARGE && zoom_now->x2max > -VERYLARGE))
+	set_explicit_range(&axis_array[SECOND_X_AXIS], zoom_now->x2min, zoom_now->x2max);
+    if (!is_3d_plot
+    && (zoom_now->y2min < VERYLARGE && zoom_now->y2max > -VERYLARGE))
+	set_explicit_range(&axis_array[SECOND_Y_AXIS], zoom_now->y2min, zoom_now->y2max);
+}
+
 #endif /* USE_MOUSE */
