@@ -46,6 +46,7 @@ extern TBOOLEAN interactive;
 extern TBOOLEAN noinputfiles;
 extern TBOOLEAN persist_cl;
 extern TBOOLEAN slow_font_startup;
+extern TBOOLEAN reading_from_dash;
 
 extern const char *user_shell;
 
@@ -75,11 +76,6 @@ void interrupt_setup(void);
 void gp_expand_tilde(char **);
 void get_user_env(void);
 
-#ifdef LINUXVGA
-void drop_privilege(void);
-void take_privilege(void);
-#endif /* LINUXVGA */
-
 #ifdef OS2
 int ExecuteMacro(char *, int);
 #endif
