@@ -672,7 +672,7 @@ show_command()
 	break;
 
     case S_MULTIPLOT:
-	fprintf(stderr,"multiplot mode is %s\n", multiplot ? "on" : "off");
+	fprintf(stderr,"multiplot mode is %s\n", (in_multiplot > 0) ? "on" : "off");
 	break;
 
     case S_TERMOPTIONS:
@@ -723,7 +723,6 @@ show_at()
 {
     (void) putc('\n', stderr);
     disp_at(temp_at(), 0);
-    c_token++;
 }
 
 
