@@ -9,12 +9,11 @@
 
 void print_table(struct curve_points * first_plot, int plot_num);
 void print_3dtable(int pcount);
-TBOOLEAN tabulate_one_line(double v[], struct value str[], int ncols);
+TBOOLEAN tabulate_one_line(struct curve_points *plot, double v[], struct value str[], int ncols);
 
 extern FILE *table_outfile;
 extern udvt_entry *table_var;
 extern TBOOLEAN table_mode;
 extern char *table_sep;
-extern struct at_type *table_filter_at;
 
 #endif /* GNUPLOT_TABULATE_H */
