@@ -39,6 +39,7 @@
 #include "gplocale.h"
 #include "internal.h"		/* for eval_reset_after_error */
 #include "misc.h"
+#include "mouse.h"		/* for zoom_reset_after_error */
 #include "multiplot.h"		/* for multiplot_reset_after_error */
 #include "plot.h"
 #include "pm3d.h"		/* for pm3d_reset_after_error */
@@ -1153,25 +1154,35 @@ int_error(int t_num, const char str[], va_dcl)
     /* reprint line if screen has been written to */
     print_line_with_error(t_num);
 
+    if (str != NULL) {
 #ifdef VA_START
-    VA_START(args, str);
+	VA_START(args, str);
 # if defined(HAVE_VFPRINTF) || _LIBC
-    vsnprintf(error_message, sizeof(error_message), str, args);
-    fprintf(stderr,"%.120s",error_message);
+	vsnprintf(error_message, sizeof(error_message), str, args);
+	fprintf(stderr,"%.120s",error_message);
 # else
-    _doprnt(str, args, stderr);
+	_doprnt(str, args, stderr);
 # endif
-    va_end(args);
+	va_end(args);
 #else
-    fprintf(stderr, str, a1, a2, a3, a4, a5, a6, a7, a8);
-    snprintf(error_message, sizeof(error_message), str, a1, a2, a3, a4, a5, a6, a7, a8);
+	fprintf(stderr, str, a1, a2, a3, a4, a5, a6, a7, a8);
+	snprintf(error_message, sizeof(error_message), str, a1, a2, a3, a4, a5, a6, a7, a8);
 #endif
+    }
 
     fputs("\n\n", stderr);
     fill_gpval_string("GPVAL_ERRMSG", error_message);
     common_error_exit();
 }
 
+/* wxwidgets based on gtk versions greater than 2.8 do not survive the
+ * LONGJMP in bail_to_command_line() if event processing has not completed.
+ * Try to detect this state and invoke additional exception handling
+ * and/or warn that a crash may be imminent.
+ */
+#ifdef WXWIDGETS
+TBOOLEAN wxt_event_processing = FALSE;
+#endif
 
 void
 common_error_exit()
@@ -1183,10 +1194,13 @@ common_error_exit()
     parse_reset_after_error();
     pm3d_reset_after_error();
     multiplot_reset_after_error();
+#ifdef USE_MOUSE
+    zoom_reset_after_error();
+#endif
     set_iterator = cleanup_iteration(set_iterator);
     plot_iterator = cleanup_iteration(plot_iterator);
     scanning_range_in_progress = FALSE;
-    inside_zoom = FALSE;
+    inside_plot_command = FALSE;
 #ifdef HAVE_LOCALE_H
     setlocale(LC_NUMERIC, "C");
 #endif
@@ -1256,7 +1270,7 @@ warn_command()
     char *message;
     c_token++;
     if ((message = try_to_get_string())) {
-	int_warn(NO_CARET, message);
+	int_warn(NO_CARET, "%s", message ? message : "");
 	free(message);
     }
 }
