@@ -37,11 +37,6 @@
 #include "syscfg.h"     /* for the prototypes */
 #include "stdfn.h"
 
-/* vax c doesn't come with strftime - watch out for redefn of RCSid */
-#ifdef VAXCRTL
-# define RCSid RCSid2
-# include "strftime.c"
-#endif
 
 /* these are needed to modify terminal characteristics */
 # ifndef VWS_XMAX
@@ -58,6 +53,9 @@
 
 static int something_in_this_file;
 
+int vms_vkid;	/* Virtual keyboard id */
+int vms_ktid;	/* key table id, for translating keystrokes */
+
 #ifdef PIPES
 
 /* (to aid porting) - how are errors dealt with */
@@ -263,12 +261,167 @@ waitpid(unsigned long int pid, int *statusp, int flags)
 
 #endif /* PIPES */
 
+/*
+ * Since vax/vms is the only remaining user of locally-coded strftime,
+ * move gnuplot's original strftime.c here.
+ */
+#ifdef VAXCRTL
+/*
+ * Implementation of strftime for systems missing this (e.g. vaxctrl)
+ *
+ * This code was written based on the NeXT strftime man-page, sample output of
+ * the function and an ANSI-C quickreference chart. This code does not use
+ * parts of any existing strftime implementation.
+ *
+ * Apparently not all format chars are implemented, but this was all I had in
+ * my documentation.
+ *
+ * (written by Alexander Lehmann)
+ */
+
+#define NOTIMEZONE
+
+static void fill(char *from, char **pto, size_t *pmaxsize);
+static void number(int num, int pad, char **pto, size_t *pmaxsize);
+
+static void
+fill(char *from, char **pto, size_t *pmaxsize)
+{
+    safe_strncpy(*pto, from, *pmaxsize);
+    if (*pmaxsize < strlen(from)) {
+	(*pto) += *pmaxsize;
+	*pmaxsize = 0;
+    } else {
+	(*pto) += strlen(from);
+	(*pmaxsize) -= strlen(from);
+    }
+}
+
+static void
+number(int num, int pad, char **pto, size_t *pmaxsize)
+{
+    char str[100];
+
+    sprintf(str, "%0*d", pad, num);
+    fill(str, pto, pmaxsize);
+}
+
+size_t
+strftime(char *s, size_t max, const char *format, const struct tm *tp)
+{
+    char *start = s;
+    size_t maxsize = max;
+
+    if (max > 0) {
+	while (*format && max > 0) {
+	    if (*format != '%') {
+		*s++ = *format++;
+		max--;
+	    } else {
+		format++;
+		switch (*format++) {
+		case 'a':	/* abbreviated weekday name */
+		    if (tp->tm_wday >= 0 && tp->tm_wday <= 6)
+			fill(abbrev_day_names[tp->tm_wday], &s, &max);
+		    break;
+		case 'A':	/* full name of the weekday */
+		    if (tp->tm_wday >= 0 && tp->tm_wday <= 6)
+			fill(full_day_names[tp->tm_wday], &s, &max);
+		    break;
+		case 'b':	/* abbreviated month name */
+		    if (tp->tm_mon >= 0 && tp->tm_mon <= 11)
+			fill(abbrev_month_names[tp->tm_mon], &s, &max);
+		    break;
+		case 'B':	/* full name of month */
+		    if (tp->tm_mon >= 0 && tp->tm_mon <= 11)
+			fill(full_month_names[tp->tm_mon], &s, &max);
+		    break;
+		case 'c':	/* locale's date and time representation */
+		    strftime(s, max, "%a %b %X %Y", tp);
+		    max -= strlen(s);
+		    s += strlen(s);
+		    break;
+		case 'd':	/* day of the month (01-31) */
+		    number(tp->tm_mday, 2, &s, &max);
+		    break;
+		case 'H':	/* hour of the day (00-23) */
+		    number(tp->tm_hour, 2, &s, &max);
+		    break;
+		case 'I':	/* hour of the day (01-12) */
+		    number((tp->tm_hour + 11) % 12 + 1, 2, &s, &max);
+		    break;
+		case 'j':	/* day of the year (001-366) */
+		    number(tp->tm_yday + 1, 3, &s, &max);
+		    break;
+		case 'm':	/* month of the year (01-12) */
+		    number(tp->tm_mon + 1, 2, &s, &max);
+		    break;
+		case 'M':	/* minute (00-59) */
+		    number(tp->tm_min, 2, &s, &max);
+		    break;
+		case 'p':	/* locale's version of AM or PM */
+		    fill(tp->tm_hour >= 6 ? "PM" : "AM", &s, &max);
+		    break;
+		case 'S':	/* seconds (00-59) */
+		    number(tp->tm_sec, 2, &s, &max);
+		    break;
+		case 'U':	/* week number of the year (00-53) with Sunday as the first day of the week */
+		    number((tp->tm_yday - (tp->tm_yday - tp->tm_wday + 7) % 7 + 7) / 7, 1, &s, &max);
+		    break;
+		case 'w':	/* weekday (Sunday = 0 to Saturday = 6) */
+		    number(tp->tm_wday, 1, &s, &max);
+		    break;
+		case 'W':	/* week number of the year (00-53) with Monday as the first day of the week */
+		    number((tp->tm_yday - (tp->tm_yday - tp->tm_wday + 8) % 7 + 7) / 7, 2, &s, &max);
+		    break;
+		case 'x':	/* locale's date representation */
+		    strftime(s, max, "%a %b %d %Y", tp);
+		    max -= strlen(s);
+		    s += strlen(s);
+		    break;
+		case 'X':	/* locale's time representation */
+#ifndef NOTIMEZONE
+		    strftime(s, max, "%H:%M:%S %Z", tp);
+#else
+		    strftime(s, max, "%H:%M:%S", tp);
+#endif
+		    max -= strlen(s);
+		    s += strlen(s);
+		    break;
+		case 'y':	/* two-digit year representation (00-99) */
+		    number(tp->tm_year % 100, 2, &s, &max);
+		    break;
+		case 'Y':	/* four-digit year representation */
+		    number(tp->tm_year + 1900, 2, &s, &max);
+		    break;
+#ifndef NOTIMEZONE
+		case 'Z':	/* time zone name */
+		    fill(tp->tm_zone, &s, &max);
+		    break;
+#endif
+		case '%':	/* percent sign */
+		default:
+		    *s++ = *(format - 1);
+		    max--;
+		    break;
+		}
+	    }
+	}
+	if (s - start < maxsize) {
+	    *s++ = '\0';
+	} else {
+	    *(s - 1) = '\0';
+	}
+    }
+    return s - start;
+}
+#endif /* VAXCRTL */
+
 /*
  * Everything below here moved out of term.c in Jan 2019
  * It is quite possible that additional #includes are needed for this to compile
  */
 
-
 # ifndef TT2$M_DECCRT3          /* VT300 not defined as of VAXC v2.4 */
 #  define TT2$M_DECCRT3 0X80000000
 # endif
@@ -429,3 +582,22 @@ fflush_binary()
 	fflush(gpoutfile);
     }
 }
+
+void
+done(int status)
+{
+    term_reset();
+    gp_exit(status);
+}
+
+void
+vms_init_screen()
+{
+    /* initialise screen management routines for command recall */
+    unsigned int ierror;
+    if (ierror = smg$create_virtual_keyboard(&vms_vkid) != SS$_NORMAL)
+	done(ierror);
+    if (ierror = smg$create_key_table(&vms_ktid) != SS$_NORMAL)
+	done(ierror);
+}
+
